@@ -64,7 +64,7 @@ printcontentxml(const char *s, FILE *fp)
 		/* XML entities */
 		case '<':  fputs("&lt;",   fp); break;
 		case '>':  fputs("&gt;",   fp); break;
-		case '\'': fputs("&apos;", fp); break;
+		case '\'': fputs("&#39;",  fp); break;
 		case '&':  fputs("&amp;",  fp); break;
 		case '"':  fputs("&quot;", fp); break;
 		default:   fputc((int)*s, fp);
@@ -108,6 +108,7 @@ printfeed(FILE *fpitems, FILE *fpin, struct feed *f)
 	ssize_t linelen;
 	FILE *fpcontent = NULL;
 	unsigned int isnew;
+	struct tm *tm;
 	time_t parsedtime;
 	int fd, r;
 
@@ -137,21 +138,25 @@ printfeed(FILE *fpitems, FILE *fpin, struct feed *f)
 		fputs("</a></h2>\n", fpitems);
 	}
 
-	fputs("<table cellpadding=\"0\" cellspacing=\"0\">\n", fpitems);
 	while ((linelen = getline(&line, &linesize, fpin)) > 0) {
 		if (line[linelen - 1] == '\n')
 			line[--linelen] = '\0';
 		if (!parseline(line, fields))
 			break;
-		/* write content */
-		if (!normalizepath(fields[FieldTitle], name, sizeof(name)))
-			continue;
-		r = snprintf(filepath, sizeof(filepath), "%s/%s.html", dirpath, name);
-		if (r == -1 || (size_t)r >= sizeof(filepath))
-			errx(1, "snprintf: path truncation: '%s/%s.html'", dirpath, name);
 
 		parsedtime = 0;
 		strtotime(fields[FieldUnixTimestamp], &parsedtime);
+		if (!(tm = localtime(&parsedtime)))
+			err(1, "localtime");
+
+		if (!normalizepath(fields[FieldTitle], name, sizeof(name)))
+			continue;
+
+		r = snprintf(filepath, sizeof(filepath), "%s/%s-%lld.html",
+		             dirpath, name, (long long)parsedtime);
+		if (r == -1 || (size_t)r >= sizeof(filepath))
+			errx(1, "snprintf: path truncation: '%s/%s-%lld.html'",
+			        dirpath, name, (long long)parsedtime);
 
 		/* content file doesn't exist yet and has error? */
 		if ((fd = open(filepath, O_CREAT | O_EXCL | O_WRONLY,
@@ -165,11 +170,18 @@ printfeed(FILE *fpitems, FILE *fpin, struct feed *f)
 			      "<link rel=\"stylesheet\" type=\"text/css\" href=\"../../style.css\" />"
 			      "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />"
 			      "</head>\n<body class=\"frame\">"
-			      "<div class=\"content\"><h2><a href=\"", fpcontent);
-			xmlencode(fields[FieldLink], fpcontent);
-			fputs("\">", fpcontent);
+			      "<div class=\"content\"><h2>", fpcontent);
+
+			if (fields[FieldLink][0]) {
+				fputs("<a href=\"", fpcontent);
+				xmlencode(fields[FieldLink], fpcontent);
+				fputs("\">", fpcontent);
+			}
 			xmlencode(fields[FieldTitle], fpcontent);
-			fputs("</a></h2>", fpcontent);
+			if (fields[FieldLink][0])
+				fputs("</a>", fpcontent);
+			fputs("</h2>", fpcontent);
+
 			/* NOTE: this prints the raw HTML of the feed, this is
 			 * potentially dangerous, it is left up to the
 			 * user / browser to trust a feed it's HTML content. */
@@ -181,7 +193,7 @@ printfeed(FILE *fpitems, FILE *fpin, struct feed *f)
 				printcontentxml(fields[FieldContent], fpcontent);
 				fputs("</pre>", fpcontent);
 			}
-			fputs("</div></body></html>", fpcontent);
+			fputs("</div></body></html>\n", fpcontent);
 
 			/* set modified and access time of file to time of item. */
 			if (parsedtime) {
@@ -203,13 +215,11 @@ printfeed(FILE *fpitems, FILE *fpin, struct feed *f)
 		totalnew += isnew;
 		f->totalnew += isnew;
 		f->total++;
-		if (isnew)
-			fputs("<tr class=\"n\">", fpitems);
-		else
-			fputs("<tr>", fpitems);
-		fputs("<td nowrap valign=\"top\">", fpitems);
-		fputs(fields[FieldTimeFormatted], fpitems);
-		fputs("</td><td nowrap valign=\"top\">", fpitems);
+
+                fprintf(fpitems, "%04d-%02d-%02d&nbsp;%02d:%02d ",
+                       tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
+                       tm->tm_hour, tm->tm_min);
+
 		if (isnew)
 			fputs("<b><u>", fpitems);
 		fputs("<a href=\"", fpitems);
@@ -219,17 +229,16 @@ printfeed(FILE *fpitems, FILE *fpin, struct feed *f)
 		fputs("</a>", fpitems);
 		if (isnew)
 			fputs("</u></b>", fpitems);
-		fputs("</td></tr>\n", fpitems);
+		fputs("\n", fpitems);
 	}
-	fputs("</table>\n", fpitems);
 }
 
 int
 main(int argc, char *argv[])
 {
 	FILE *fpindex, *fpitems, *fpmenu, *fp;
-	int showsidebar = (argc > 1);
-	int i;
+	char *name;
+	int i, showsidebar = (argc > 1);
 	struct feed *f;
 
 	if (pledge("stdio rpath wpath cpath fattr", NULL) == -1)
@@ -252,7 +261,7 @@ main(int argc, char *argv[])
 		err(1, "fopen: items.html");
 	fputs("<html><head><link rel=\"stylesheet\" type=\"text/css\" href=\"../style.css\" />"
 	      "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" /></head>"
-	      "<body class=\"frame\"><div id=\"items\">", fpitems);
+	      "<body class=\"frame\"><div id=\"items\"><pre>", fpitems);
 
 	if (argc == 1) {
 		if (!(feeds[0] = calloc(1, sizeof(struct feed))))
@@ -263,7 +272,8 @@ main(int argc, char *argv[])
 		for (i = 1; i < argc; i++) {
 			if (!(feeds[i - 1] = calloc(1, sizeof(struct feed))))
 				err(1, "calloc");
-			feeds[i - 1]->name = xbasename(argv[i]);
+			name = ((name = strrchr(argv[i], '/'))) ? name + 1 : argv[i];
+			feeds[i - 1]->name = name;
 
 			if (!(fp = fopen(argv[i], "r")))
 				err(1, "fopen: %s", argv[i]);
@@ -273,7 +283,7 @@ main(int argc, char *argv[])
 			fclose(fp);
 		}
 	}
-	fputs("\n</div></body>\n</html>", fpitems); /* div items */
+	fputs("</pre>\n</div></body>\n</html>\n", fpitems); /* div items */
 
 	if (showsidebar) {
 		fputs("<html><head>"
@@ -297,7 +307,7 @@ main(int argc, char *argv[])
 				fputs("</u></b>", fpmenu);
 			fputs("</a><br/>\n", fpmenu);
 		}
-		fputs("</div></body></html>", fpmenu);
+		fputs("</div></body></html>\n", fpmenu);
 	}
 	fputs("<!DOCTYPE html><html><head>\n\t<title>Newsfeed (", fpindex);
 	fprintf(fpindex, "%lu", totalnew);
@@ -315,7 +325,7 @@ main(int argc, char *argv[])
 	      "\t\t<frame name=\"content\" target=\"content\">\n"
 	      "\t</frameset>\n"
 	      "</frameset>\n"
-	      "</html>", fpindex);
+	      "</html>\n", fpindex);
 
 	fclose(fpitems);
 	fclose(fpmenu);
