@@ -17,11 +17,12 @@ static void
 printfeed(FILE *fp, struct feed *f)
 {
 	char *fields[FieldLast];
+	struct tm *tm;
 	time_t parsedtime;
 	unsigned int islink, isnew;
 	ssize_t linelen;
 
-	if (f->name[0] != '\0') {
+	if (f->name[0]) {
 		fputs("<h2 id=\"", stdout);
 		xmlencode(f->name, stdout);
 		fputs("\"><a href=\"#", stdout);
@@ -30,30 +31,28 @@ printfeed(FILE *fp, struct feed *f)
 		xmlencode(f->name, stdout);
 		fputs("</a></h2>\n", stdout);
 	}
-	fputs("<table cellpadding=\"0\" cellspacing=\"0\">\n", stdout);
 
 	while ((linelen = getline(&line, &linesize, fp)) > 0) {
 		if (line[linelen - 1] == '\n')
 			line[--linelen] = '\0';
 		if (!parseline(line, fields))
 			break;
+
 		parsedtime = 0;
 		strtotime(fields[FieldUnixTimestamp], &parsedtime);
+	        if (!(tm = localtime(&parsedtime)))
+			err(1, "localtime");
 
 		isnew = (parsedtime >= comparetime) ? 1 : 0;
-		islink = (fields[FieldLink][0] != '\0') ? 1 : 0;
+		islink = fields[FieldLink][0] ? 1 : 0;
 
 		totalnew += isnew;
 		f->totalnew += isnew;
 		f->total++;
 
-		if (isnew)
-			fputs("<tr class=\"n\">", stdout);
-		else
-			fputs("<tr>", stdout);
-		fputs("<td nowrap valign=\"top\">", stdout);
-		fputs(fields[FieldTimeFormatted], stdout);
-		fputs("</td><td nowrap valign=\"top\">", stdout);
+	        fprintf(stdout, "%04d-%02d-%02d&nbsp;%02d:%02d ",
+		        tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
+		        tm->tm_hour, tm->tm_min);
 		if (isnew)
 			fputs("<b><u>", stdout);
 		if (islink) {
@@ -66,15 +65,15 @@ printfeed(FILE *fp, struct feed *f)
 			fputs("</a>", stdout);
 		if (isnew)
 			fputs("</u></b>", stdout);
-		fputs("</td></tr>\n", stdout);
+		fputs("\n", stdout);
 	}
-	fputs("</table>\n", stdout);
 }
 
 int
 main(int argc, char *argv[])
 {
 	struct feed *f;
+	char *name;
 	FILE *fp;
 	int i;
 
@@ -101,6 +100,7 @@ main(int argc, char *argv[])
 		fputs("\t\t<div id=\"items\">\n", stdout);
 	else
 		fputs("\t\t<div id=\"items\" class=\"nosidebar\">\n", stdout);
+	fputs("<pre>", stdout);
 
 	if (argc == 1) {
 		if (!(feeds[0] = calloc(1, sizeof(struct feed))))
@@ -113,8 +113,8 @@ main(int argc, char *argv[])
 		for (i = 1; i < argc; i++) {
 			if (!(feeds[i - 1] = calloc(1, sizeof(struct feed))))
 				err(1, "calloc");
-			feeds[i - 1]->name = xbasename(argv[i]);
-
+			name = ((name = strrchr(argv[i], '/'))) ? name + 1 : argv[i];
+			feeds[i - 1]->name = name;
 			if (!(fp = fopen(argv[i], "r")))
 				err(1, "fopen: %s", argv[i]);
 			printfeed(fp, feeds[i - 1]);
@@ -123,7 +123,7 @@ main(int argc, char *argv[])
 			fclose(fp);
 		}
 	}
-	fputs("</div>\n", stdout); /* div items */
+	fputs("</pre>\n</div>\n", stdout); /* div items */
 
 	if (showsidebar) {
 		fputs("\t<div id=\"sidebar\">\n\t\t<ul>\n", stdout);
@@ -147,7 +147,7 @@ main(int argc, char *argv[])
 		fputs("\t\t</ul>\n\t</div>\n", stdout);
 	}
 
-	fprintf(stdout, "\t</body>\n\t<title>Newsfeed (%lu)</title>\n</html>",
+	fprintf(stdout, "\t</body>\n\t<title>Newsfeed (%lu)</title>\n</html>\n",
 	        totalnew);
 
 	return 0;
