@@ -35,7 +35,7 @@ read:
 			case 't':  fputc('\t', fp); break;
 			case 'n':
 				fputc('\n', fp);
-				for (s++; *s && *s == '>'; s++)
+				for (s++; *s == '>'; s++)
 					fputc('>', fp);
 				/* escape "From ", mboxrd-style. */
 				if (!strncmp(s, "From ", 5))
@@ -45,7 +45,7 @@ read:
 			break;
 		case '\n':
 			fputc((int)*s, fp);
-			for (s++; *s && *s == '>'; s++)
+			for (s++; *s == '>'; s++)
 				fputc('>', fp);
 			/* escape "From ", mboxrd-style. */
 			if (!strncmp(s, "From ", 5))
@@ -143,9 +143,8 @@ main(int argc, char *argv[])
 		for (i = 1; i < argc; i++) {
 			if (!(fp = fopen(argv[i], "r")))
 				err(1, "fopen: %s", argv[i]);
-			name = xbasename(argv[i]);
+			name = ((name = strrchr(argv[i], '/'))) ? name + 1 : argv[i];
 			printfeed(fp, name);
-			free(name);
 			if (ferror(fp))
 				err(1, "ferror: %s", argv[i]);
 			fclose(fp);
