@@ -1,5 +1,6 @@
 #include <ctype.h>
 #include <err.h>
+#include <locale.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -15,6 +16,7 @@ static void
 printfeed(FILE *fp, const char *feedname)
 {
 	char *fields[FieldLast];
+	struct tm *tm;
 	time_t parsedtime;
 	ssize_t linelen;
 
@@ -26,6 +28,8 @@ printfeed(FILE *fp, const char *feedname)
 
 		parsedtime = 0;
 		strtotime(fields[FieldUnixTimestamp], &parsedtime);
+	        if (!(tm = localtime(&parsedtime)))
+			err(1, "localtime");
 
 		if (parsedtime >= comparetime)
 			fputs("N ", stdout);
@@ -33,8 +37,11 @@ printfeed(FILE *fp, const char *feedname)
 			fputs("  ", stdout);
 
 		if (feedname[0])
-			printf("%-15.15s ", feedname);
-		printf("%-30.30s ", fields[FieldTimeFormatted]);
+			printf("%-15.15s  ", feedname);
+
+	        fprintf(stdout, "%04d-%02d-%02d %02d:%02d  ",
+		        tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
+		        tm->tm_hour, tm->tm_min);
 		printutf8pad(stdout, fields[FieldTitle], 70, ' ');
 		printf(" %s\n", fields[FieldLink]);
 	}
@@ -47,6 +54,11 @@ main(int argc, char *argv[])
 	char *name;
 	int i;
 
+	if (pledge("stdio rpath", NULL) == -1)
+		err(1, "pledge");
+
+	setlocale(LC_CTYPE, "");
+
 	if (pledge(argc == 1 ? "stdio" : "stdio rpath", NULL) == -1)
 		err(1, "pledge");
 
@@ -61,9 +73,8 @@ main(int argc, char *argv[])
 		for (i = 1; i < argc; i++) {
 			if (!(fp = fopen(argv[i], "r")))
 				err(1, "fopen: %s", argv[i]);
-			name = xbasename(argv[i]);
+			name = ((name = strrchr(argv[i], '/'))) ? name + 1 : argv[i];
 			printfeed(fp, name);
-			free(name);
 			if (ferror(fp))
 				err(1, "ferror: %s", argv[i]);
 			fclose(fp);
