@@ -3,7 +3,6 @@
 #include <ctype.h>
 #include <err.h>
 #include <errno.h>
-#include <libgen.h>
 #include <limits.h>
 #include <stdarg.h>
 #include <stdio.h>
@@ -15,14 +14,6 @@
 
 #include "util.h"
 
-#ifndef USE_PLEDGE
-int
-pledge(const char *promises, const char *paths[])
-{
-	return 0;
-}
-#endif
-
 int
 parseuri(const char *s, struct uri *u, int rel)
 {
@@ -78,10 +69,11 @@ parseuri(const char *s, struct uri *u, int rel)
 			return -1; /* port too long */
 		memcpy(u->port, p, i);
 		u->port[i] = '\0';
-		/* check for valid port */
+		/* check for valid port: range 1 - 65535 */
 		errno = 0;
 		l = strtoul(u->port, &endptr, 10);
-		if (errno || *endptr != '\0' || !l || l > 65535)
+		if (errno || u->port[0] == '\0' || *endptr ||
+		    !l || l > 65535)
 			return -1;
 		p = &p[i];
 	}
@@ -100,10 +92,38 @@ readpath:
 	return 0;
 }
 
+static int
+encodeuri(char *buf, size_t bufsiz, const char *s)
+{
+	static const char *table = "0123456789ABCDEF";
+	size_t i, b;
+
+	for (i = 0, b = 0; s[i]; i++) {
+		if ((int)s[i] == ' ' ||
+		    (unsigned char)s[i] > 127 ||
+		    iscntrl((int)s[i])) {
+			if (b + 3 >= bufsiz)
+				return -1;
+			buf[b++] = '%';
+			buf[b++] = table[((uint8_t)s[i] >> 4) & 15];
+			buf[b++] = table[(uint8_t)s[i] & 15];
+		} else if (b < bufsiz) {
+			buf[b++] = s[i];
+		} else {
+			return -1;
+		}
+	}
+	if (b >= bufsiz)
+		return -1;
+	buf[b] = '\0';
+
+	return 0;
+}
+
 /* Get absolute uri; if `link` is relative use `base` to make it absolute.
  * the returned string in `buf` is uri encoded, see: encodeuri(). */
 int
-absuri(const char *link, const char *base, char *buf, size_t bufsiz)
+absuri(char *buf, size_t bufsiz, const char *link, const char *base)
 {
 	struct uri ulink, ubase;
 	char tmp[4096], *host, *p, *port;
@@ -154,37 +174,7 @@ absuri(const char *link, const char *base, char *buf, size_t bufsiz)
 	if (strlcat(tmp, ulink.path, sizeof(tmp)) >= sizeof(tmp))
 		return -1;
 
-	return encodeuri(tmp, buf, bufsiz);
-}
-
-int
-encodeuri(const char *s, char *buf, size_t bufsiz)
-{
-	static const char *table = "0123456789ABCDEF";
-	size_t i, b;
-
-	if (!bufsiz)
-		return -1;
-	for (i = 0, b = 0; s[i]; i++) {
-		if ((int)s[i] == ' ' ||
-		    (unsigned char)s[i] > 127 ||
-		    iscntrl((int)s[i])) {
-			if (b + 3 >= bufsiz)
-				return -1;
-			buf[b++] = '%';
-			buf[b++] = table[((uint8_t)s[i] >> 4) & 15];
-			buf[b++] = table[(uint8_t)s[i] & 15];
-		} else {
-			if (b >= bufsiz)
-				return -1;
-			buf[b++] = s[i];
-		}
-	}
-	if (b >= bufsiz)
-		return -1;
-	buf[b] = '\0';
-
-	return 0;
+	return encodeuri(buf, bufsiz, tmp);
 }
 
 /* Read a field-separated line from 'fp',
@@ -214,17 +204,19 @@ parseline(char *line, char *fields[FieldLast])
 	return i;
 }
 
-/* Parse time to time_t, assumes time_t is signed. */
+/* Parse time to time_t, assumes time_t is signed, ignores fractions. */
 int
 strtotime(const char *s, time_t *t)
 {
-	long l;
+	long long l;
 	char *e;
 
 	errno = 0;
-	l = strtol(s, &e, 10);
-	if (*s == '\0' || *e != '\0')
+	l = strtoll(s, &e, 10);
+	if (errno || *s == '\0' || *e)
 		return -1;
+	/* NOTE: assumes time_t is 64-bit on 64-bit platforms:
+	         long long (atleast 32-bit) to time_t. */
 	if (t)
 		*t = (time_t)l;
 
@@ -239,7 +231,7 @@ xmlencode(const char *s, FILE *fp)
 		switch(*s) {
 		case '<':  fputs("&lt;",   fp); break;
 		case '>':  fputs("&gt;",   fp); break;
-		case '\'': fputs("&apos;", fp); break;
+		case '\'': fputs("&#39;",  fp); break;
 		case '&':  fputs("&amp;",  fp); break;
 		case '"':  fputs("&quot;", fp); break;
 		default:   fputc(*s, fp);
@@ -247,46 +239,32 @@ xmlencode(const char *s, FILE *fp)
 	}
 }
 
-/* Some implementations of basename(3) return a pointer to a static
- * internal buffer (OpenBSD). Others modify the contents of `path` (POSIX).
- * This is a wrapper function that is compatible with both versions.
- * The program will error out if basename(3) failed, this can only happen
- * with the OpenBSD version. */
-char *
-xbasename(const char *path)
-{
-	char *p, *b;
-
-	if (!(p = strdup(path)))
-		err(1, "strdup");
-	if (!(b = basename(p)))
-		err(1, "basename");
-	if (!(b = strdup(b)))
-		err(1, "strdup");
-	free(p);
-	return b;
-}
-
 /* print `len' columns of characters. If string is shorter pad the rest
  * with characters `pad`. */
 void
 printutf8pad(FILE *fp, const char *s, size_t len, int pad)
 {
 	wchar_t w;
-	size_t n = 0, i;
-	int r;
+	size_t col = 0, i, slen;
+	int rl, wc;
 
-	for (i = 0; *s && n < len; i++, s++) {
-		if (ISUTF8(*s)) {
-			if ((r = mbtowc(&w, s, 4)) == -1)
-				break;
-			if ((r = wcwidth(w)) == -1)
-				r = 1;
-			n += (size_t)r;
+	if (!len)
+		return;
+
+	slen = strlen(s);
+	for (i = 0; i < slen && col < len + 1; i += rl) {
+		if ((rl = mbtowc(&w, &s[i], slen - i < 4 ? slen - i : 4)) <= 0)
+			break;
+		if ((wc = wcwidth(w)) == -1)
+			wc = 1;
+		col += (size_t)wc;
+		if (col >= len && s[i + rl]) {
+			fputs("\xe2\x80\xa6", fp);
+			break;
 		}
-		putc(*s, fp);
+		fwrite(&s[i], 1, rl, fp);
 	}
-	for (; n < len; n++)
+	for (; col < len; col++)
 		putc(pad, fp);
 }
 
