@@ -1,5 +1,10 @@
 #include <stdint.h>
 #include <time.h>
+#ifdef USE_PLEDGE
+#include <unistd.h>
+#else
+#define pledge(p1,p2) 0
+#endif
 
 #undef strlcat
 size_t strlcat(char *, const char *, size_t);
@@ -7,7 +12,6 @@ size_t strlcat(char *, const char *, size_t);
 size_t strlcpy(char *, const char *, size_t);
 
 #define ISUTF8(c) (((c) & 0xc0) != 0x80)
-#define LEN(x) (sizeof (x) / sizeof *(x))
 
 /* feed info */
 struct feed {
@@ -27,25 +31,16 @@ struct uri {
 };
 
 enum {
-	FieldUnixTimestamp = 0, FieldTimeFormatted, FieldTitle,
-	FieldLink, FieldContent, FieldContentType, FieldId, FieldAuthor,
-	FieldFeedType, FieldLast
+	FieldUnixTimestamp = 0, FieldTitle, FieldLink,
+	FieldContent, FieldContentType, FieldId, FieldAuthor, FieldLast
 };
 
-int     absuri(const char *, const char *, char *, size_t);
-int     encodeuri(const char *, char *, size_t);
+int     absuri(char *, size_t, const char *, const char *);
 size_t  parseline(char *, char *[FieldLast]);
 int     parseuri(const char *, struct uri *, int);
 void    printutf8pad(FILE *, const char *, size_t, int);
 int     strtotime(const char *, time_t *);
-char *  xbasename(const char *);
 void    xmlencode(const char *, FILE *);
 
-#ifdef USE_PLEDGE
-#include <unistd.h>
-#else
-int     pledge(const char *, const char *[]);
-#endif
-
 #define ROT32(x, y) ((x << y) | (x >> (32 - y)))
 uint32_t murmur3_32(const char *, uint32_t, uint32_t);
